/*******************************************************************************
 * Copyright IBM Corp. and others 1991
 *
 * This program and the accompanying materials are made available under
 * the terms of the Eclipse Public License 2.0 which accompanies this
 * distribution and is available at https://www.eclipse.org/legal/epl-2.0/
 * or the Apache License, Version 2.0 which accompanies this distribution and
 * is available at https://www.apache.org/licenses/LICENSE-2.0.
 *
 * This Source Code may also be made available under the following
 * Secondary Licenses when the conditions for such availability set
 * forth in the Eclipse Public License, v. 2.0 are satisfied: GNU
 * General Public License, version 2 with the GNU Classpath
 * Exception [1] and GNU General Public License, version 2 with the
 * OpenJDK Assembly Exception [2].
 *
 * [1] https://www.gnu.org/software/classpath/license.html
 * [2] https://openjdk.org/legal/assembly-exception.html
 *
 * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
 *******************************************************************************/
package com.ibm.j9ddr;

import java.util.Collection;

import com.ibm.j9ddr.StructureReader.StructureDescriptor;

public interface IVMData {
	/**
	 * Bootstraps into a class by absolute classname
	 */
	public void bootstrap(String classname, Object ...userData) throws ClassNotFoundException;
	
	/**
	 * Bootstraps into a class by classname relative to the versioned package (e.g. com.ibm.j9ddr.vm23)
	 */
	public void bootstrapRelative(String relativeClassname, Object ...userData) throws ClassNotFoundException;
	
	public Collection<StructureDescriptor> getStructures();

	/**
	 * Returns the J9DDR classloader associated with this VM
	 */
	public J9DDRClassLoader getClassLoader();
	
	/**
	 * The version of the VM
	 * @return string representation of the VM version
	 */
	public String getVersion();
}
