/*******************************************************************************
 * Copyright (c) 2018, 2018 Felipe Pontes
 *
 * This program and the accompanying materials are made available under
 * the terms of the Eclipse Public License 2.0 which accompanies this
 * distribution and is available at https://www.eclipse.org/legal/epl-2.0/
 * or the Apache License, Version 2.0 which accompanies this distribution and
 * is available at https://www.apache.org/licenses/LICENSE-2.0.
 *
 * This Source Code may also be made available under the following
 * Secondary Licenses when the conditions for such availability set
 * forth in the Eclipse Public License, v. 2.0 are satisfied: GNU
 * General Public License, version 2 with the GNU Classpath
 * Exception [1] and GNU General Public License, version 2 with the
 * OpenJDK Assembly Exception [2].
 *
 * [1] https://www.gnu.org/software/classpath/license.html
 * [2] https://openjdk.org/legal/assembly-exception.html
 *
 * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
 *******************************************************************************/
package org.openj9.test.reflect;

import org.openj9.resources.reflect.B;
import org.testng.annotations.Test;
import org.testng.Assert;
import java.lang.reflect.Method;

public class GetDeclaredMethodsTests {

	@Test(groups = { "level.sanity" })
	public void testGetDeclaredMethodsAfterNewInstance() {
		try {
			B.class.newInstance();
		} catch (ExceptionInInitializerError e) {
		} catch (Exception e) {
		} finally {
			Method[] declaredMethods = B.class.getDeclaredMethods();
			int expectedMethodsListSize = 1;
			int declaredMethodsListSize = declaredMethods.length;
			Assert.assertEquals(expectedMethodsListSize, declaredMethodsListSize, "wrong methods list size");
			String expectedMethodSignature = "public void org.openj9.resources.reflect.B.m()";
			String actualMethodSignature = declaredMethods[0].toString();
			Assert.assertEquals(expectedMethodSignature, actualMethodSignature, "wrong signatures for method");
		}
	}

}
